#include "extension.h"

#include "CDetour/detours.h"

#define GAMEDATA_FILE "witchfix"

CDetour *Detour_WitchAttack__Create = NULL;
CDetour *Detour_WitchAttack__GetVictim = NULL;

IGameConfig *g_pGameConf = NULL;
IServerGameEnts *gameents = NULL;

int g_WitchACharasterOffset = -1;

Witchfix g_Witchfix;		/**< Global singleton for extension's main interface */
SMEXT_LINK(&g_Witchfix);

DETOUR_DECL_MEMBER1(WitchAttack__WitchAttack, void* ,CBaseEntity*,pEntity)
{
	int client=gamehelpers->IndexOfEdict(gameents->BaseEntityToEdict((CBaseEntity*)pEntity));
	IGamePlayer *player = playerhelpers->GetGamePlayer(client);
	if (player)
	{
		g_pSM->LogMessage(myself,"WitchAttack created for client %s.",player->GetName());	
	}
	else
	{
		g_pSM->LogMessage(myself,"WitchAttack created for %d entity.",client);
	}

	void*result=DETOUR_MEMBER_CALL(WitchAttack__WitchAttack)(pEntity);

	DWORD*CharId=((DWORD *)this + g_WitchACharasterOffset);
	g_pSM->LogMessage(myself,"WitchAttack char=%d.",*CharId);
	*CharId=4;

	return result;
}

DETOUR_DECL_MEMBER0(WitchAttack__GetVictim, void*)
{
	void*result = DETOUR_MEMBER_CALL(WitchAttack__GetVictim)();

	DWORD*CharId = ((DWORD *)this + g_WitchACharasterOffset);
	*CharId = 4;

	return result;
}

bool Witchfix::SDK_OnMetamodLoad( ISmmAPI *ismm, char *error, size_t maxlength, bool late )
{
	size_t maxlen=maxlength;
	GET_V_IFACE_ANY(GetServerFactory, gameents, IServerGameEnts, INTERFACEVERSION_SERVERGAMEENTS);

	return true;
}

bool Witchfix::SDK_OnLoad( char *error, size_t maxlength, bool late )
{
	char conf_error[255];
	if (!gameconfs->LoadGameConfigFile(GAMEDATA_FILE, &g_pGameConf, conf_error, sizeof(conf_error))){
		if (!strlen(conf_error)){
			snprintf(error, maxlength, "Could not read witchfix.txt: %s", conf_error);
		}
		return false;
	}

	g_pGameConf->GetOffset("WitchAttackCharaster",&g_WitchACharasterOffset);

	if (!SetupHooks()) return false;

	return true;
}

void Witchfix::SDK_OnUnload()
{
	//remove hooks
	RemoveHooks();

	gameconfs->CloseGameConfigFile(g_pGameConf);
}

bool Witchfix::SetupHooks()
{
	CDetourManager::Init(g_pSM->GetScriptingEngine(), g_pGameConf);	

	//witch fix hooks
	Detour_WitchAttack__Create=DETOUR_CREATE_MEMBER(WitchAttack__WitchAttack, "WitchAttack::WitchAttack");
	if (Detour_WitchAttack__Create) {
		Detour_WitchAttack__Create->EnableDetour();
	} else {
		g_pSM->LogError(myself,"Cannot find signature of WitchAttack::WitchAttack");
		RemoveHooks();
		return false;
	}
	Detour_WitchAttack__GetVictim = DETOUR_CREATE_MEMBER(WitchAttack__GetVictim, "WitchAttack::GetVictim");
	if (Detour_WitchAttack__GetVictim) {
		Detour_WitchAttack__GetVictim->EnableDetour();
	}
	else {
		g_pSM->LogError(myself, "Cannot find signature of WitchAttack::GetVictim");
		RemoveHooks();
		return false;
	}

	return true;
}

void Witchfix::RemoveHooks()
{
	if (Detour_WitchAttack__Create){
		Detour_WitchAttack__Create->Destroy();
		Detour_WitchAttack__Create=NULL;
	}
	if (Detour_WitchAttack__GetVictim){
		Detour_WitchAttack__GetVictim->Destroy();
		Detour_WitchAttack__GetVictim = NULL;
	}
}

